# -*- encoding: utf-8 -*-

"""
This is Wombat, an operations and maintenance tool for Erlang systems
© 2015, Erlang Solutions Ltd
"""

from __future__ import absolute_import

from wombat.wombat import Wombat
from wombat.arg_parse import ArgParse
import argparse
import sys
import os
HEADER = """
define host{{
        name                            wombat-host    ; The name of this host template
        notifications_enabled           1       ; Host notifications are enabled
        event_handler_enabled           1       ; Host event handler is enabled
        flap_detection_enabled          1       ; Flap detection is enabled
        failure_prediction_enabled      1       ; Failure prediction is enabled
        process_perf_data               1       ; Process performance data
        retain_status_information       1       ; Retain status information across program restarts
        retain_nonstatus_information    1       ; Retain non-status information across program restarts
        check_command                   check-host-alive
        max_check_attempts              10
        notification_interval           0
        notification_period             24x7
        notification_options            d,u,r
        contact_groups                  admins
        register                        0       ; DONT REGISTER THIS DEFINITION - ITS NOT A REAL HOST, JUST A TEMPLATE!
        }}

define host{{
        name                            erlang-host    ; The name of this host template
        notifications_enabled           1       ; Host notifications are enabled
        event_handler_enabled           1       ; Host event handler is enabled
        flap_detection_enabled          1       ; Flap detection is enabled
        failure_prediction_enabled      1       ; Failure prediction is enabled
        process_perf_data               1       ; Process performance data
        retain_status_information       1       ; Retain status information across program restarts
        retain_nonstatus_information    1       ; Retain non-status information across program restarts
        check_command                   check-host-alive
        max_check_attempts              10
        notification_interval           0
        notification_period             24x7
        notification_options            d,u,r
        contact_groups                  admins
        register                        0       ; DONT REGISTER THIS DEFINITION - ITS NOT A REAL HOST, JUST A TEMPLATE!
        }}
        
define service{{
        name                            wombat-oam ; The 'name' of this service template
        service_description             Wombat OAM
        _wombat_connection              {0} --user=admin --password=admin -H $_SERVICEWOMBAT_ADDRESS$
        check_command                   check-wombat-alive
        servicegroups                   wombat-managed
        
        active_checks_enabled           1       ; Active service checks are enabled
        passive_checks_enabled          1       ; Passive service checks are enabled/accepted
        parallelize_check               1       ; Active service checks should be parallelized (disabling this can lead to major performance problems)
        obsess_over_service             1       ; We should obsess over this service (if necessary)
        check_freshness                 0       ; Default is to NOT check service 'freshness'
        notifications_enabled           1       ; Service notifications are enabled
        event_handler_enabled           1       ; Service event handler is enabled
        flap_detection_enabled          1       ; Flap detection is enabled
        failure_prediction_enabled      1       ; Failure prediction is enabled
        process_perf_data               1       ; Process performance data
        retain_status_information       1       ; Retain status information across program restarts
        retain_nonstatus_information    1       ; Retain non-status information across program restarts
        notification_interval           0       ; Only send notifications on status change by default.
        is_volatile                     0
        check_period                    24x7
        normal_check_interval           5
        retry_check_interval            1
        max_check_attempts              4
        notification_period             24x7
        notification_options            w,u,c,r
        contact_groups                  admins
        register                        0       ; DONT REGISTER THIS DEFINITION - ITS NOT A REAL SERVICE, JUST A TEMPLATE!
        }}
        
define service{{
        name                            erlang-node ; The 'name' of this service template
        service_description             Erlang Node
        check_command                   check-wombat-node
        servicegroups                   wombat-managed
        use                             wombat-oam
        register                        0
        }}

define servicegroup{{
        servicegroup_name              wombat-managed
        alias                          wombat-managed
        }}

define command{{
    command_name check-wombat-alive
    command_line wombat_check $_SERVICEWOMBAT_CONNECTION$ check_wombat
}}

define command{{
    command_name check-wombat-node
    command_line wombat_check $_SERVICEWOMBAT_CONNECTION$ check_node $_SERVICEWOMBAT_UUID$
}}


"""

class NagConf:
    def __init__(self, what):
        self.ty = what
        self.fields = {}
    def add(self, k, v):
        self.fields[k] = v
        return self
    def print_cfg(self, f = sys.stdout):
        f.write("define {}{{\n".format(self.ty))
        for k in sorted(self.fields):
            f.write("        {!s:<20} {}\n".format(k, self.fields[k]))
        f.write("}\n\n")

def make_wombat_conf(w, a):
    port_opt = ''
    if a.https:
        port_opt = '-s '
    port_opt += '-p {0.port}'.format(a)
    conn_opts = port_opt + ' --password={0.password} --user={0.user} -H {0.host}'.format(a)
    h = NagConf('host').add('use', 'wombat-host').add('host_name', w.host)
    s = NagConf('service').add('use', 'wombat-oam') \
            .add('host_name', w.host) \
            .add('_wombat_connection', conn_opts)
    return port_opt, conn_opts, [h,s]

def make_config(w, a):
    port_opt, conn_opts, nodecfg = make_wombat_conf(w, a)
    nodes = w.get("topo/node")
    for n in nodes:
        H = n['name'].split('@')[1]
        ch = NagConf('host').add('use', 'erlang-host').add('host_name', H)
        cs = NagConf('service').add('use', 'erlang-node') \
              .add('_wombat_uuid',n['id']) \
              .add('host_name',H) \
              .add('_wombat_connection', conn_opts)
        nodecfg += [ch,cs]
    print(HEADER.format(port_opt))
    for c in nodecfg:
        c.print_cfg()

def main():
    a, rest = ArgParse().parse_known_args()
    w = Wombat(a.host, a.port, a.user, a.password, https=a.https)
    make_config(w, a)

# Main program 
if __name__ == '__main__':
    main()
