# -*- encoding: utf-8 -*-
#

"""
This is Wombat, an operations and maintenance tool for Erlang systems
© 2015, Erlang Solutions Ltd
"""

from __future__ import absolute_import

from wombat.wombat import Wombat
from wombat.arg_parse import ArgParse
import sys

OK, WARNING, CRITICAL, UNKNOWN = 0, 1, 2, 3
state_names = ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN']

UP, DOWN, UNREACHABLE = 0, 1, 2


def plugin_ret(Name, State, Msg):
    return "{} {} {}\n".format(Name, state_names[State], Msg)

def plugin_exit(Name, State, Msg):
    sys.stdout.write(plugin_ret(Name, State, Msg))
    sys.exit(State)

def fmt_alarms(As):
    s = '\n'
    for a in As:
        s += '\n{:<20} {:<20} {:<20}'.format(
                "ACKNOWLEDGED" if a.ackInfo else '',
                a.severity,
                a.alarmId)
    return s

class WombatService(object):
    def __init__(self, w):
        self.wombat = w

    def ping(self):
        try:
            text = self.wombat.get('help/about')['text']
            WVer = text.split('\n')[1].split()[1]
            alarms = list(self.wombat.enum_alarms())
            return (OK,
                    WVer + " - {} alarm(s) present - "
                     .format(len(alarms)) + self.wombat.dashboard_url)
        except Exception as e:
            return (CRITICAL, str(e))

class WombatNode(object):
    def __init__(self, w, uu):
        self.wombat = w
        self.uuid = uu
        self.node = self.wombat.get_node(self.uuid, field = 'id')

    def ping(self):
        try:
            n = self.wombat.get('topo/node/' + self.uuid)
            if n['state'] == 'UP':
               alarms = list(self.node.enum_alarms())
               if len(alarms) == 0:
                   return (OK,
                       "{} is UP - no alarms".format(n['name']))
               else:
                   return (WARNING,
                              "{} is UP - {} alarm(s) - ".format(n['name'], len(alarms))
                              + self.wombat.dashboard_url
                              + fmt_alarms(alarms))
            else:
               return (CRITICAL, "{} is {}".format(n['name'], n['state']))
        except Exception as e:
            return (CRITICAL, str(e))

def main():
    a, rest = ArgParse().parse_known_args()
    try:
        w = Wombat(a.host, a.port, a.user, a.password, https=a.https)

        if rest == ['check_wombat']:
            st, msg = WombatService(w).ping()
        elif len(rest) == 2 and rest[0] == 'check_node':
            uuid = rest[1]
            st, msg = WombatNode(w, uuid).ping()
        else:
            st, msg = WARNING, "bad arguments to script: {}".format(sys.argv)
        plugin_exit('WOMBAT', st, msg)
    except Exception as e:
       plugin_exit('WOMBAT', CRITICAL, str(e))

# Main program 
if __name__ == '__main__':
    main()
