# -*- encoding: utf-8 -*-
#

"""
This is Wombat, an operations and maintenance tool for Erlang systems
© 2015, Erlang Solutions Ltd
"""

from __future__ import absolute_import

from wombat.wombat_object import WombatObject

class NodeFamily(WombatObject):
    """This class represents a node family in Wombat."""

    def __init__(self, wombat, id, data = None):
        WombatObject.__init__(self, wombat, id, 'topo/node-family/{}')

    def get_nodes(self):
        """Return a list of Node objects in this family."""
        return [Node(self.wombat, node_id) for node_id in self.nodes]

    def get_metric_info(self):
        """Get metadata for all known metrics on the node family."""
        return self.wombat.get_metric_info(node_family = self.id)

    def get_metric(self, metric_id, from_time, to_time, series = 1):
        """Fetch metric samples for a metric in the given time interval.

        Only counters and gauges can be queried on the node family
        level. The return value is a dictionary mapping node ids to
        list of samples.

        """

        # The metric type doesn't matter, it could be eithe counter or
        # gauge
        return self.wombat.get_metric(metric_id, 'counter', from_time, to_time,
                                      series, node_family = self.id)

    def enum_logs(self, originator = None):
        """Return a generator for enumerating logs from the node
        family.

        It is possible to enumerate only logs coming from a given
        originator.

        """
        return self.wombat.enum_logs(node_family = self.id,
                                     originator = originator)

    def get_log_originators(self):
        """Return the list of all log originators available for filtering the
        notifications on the node family.

        """
        return self.wombat.get_log_originators(node_family = self.id)

    def enum_alarms(self, source = None, following = None):
        """Return a generator for enumerating alarms on the node family.

        When no arguments are provided, all active alarms on the node
        family are considered. But it is possible to filter for a
        specific alarm source or to enumerate only alarms following a
        given event.

        """
        return self.wombat.enum_alarms(node_family = self.id,
                                       source = source,
                                       following = following)

    def get_alarm_sources(self):
        """Return the list of all alarm sources available for
        filtering the alarms on the node family.

        """
        return self.wombat.get_alarm_sources(node_family = self.id)

    def delete(self):
        """Remove the node family from Wombat.

        This function fails if there are any nodes left in the node
        family.

        """
        self.wombat.delete_node_family(node_family = self.id)
