# -*- encoding: utf-8 -*-
#

"""
This is Wombat, an operations and maintenance tool for Erlang systems
© 2015, Erlang Solutions Ltd
"""

import sys
import json
import base64
if sys.version_info[0] == 2:
    # Import Python 2.x classes
    from urllib2 import \
        HTTPPasswordMgrWithDefaultRealm, \
        HTTPBasicAuthHandler, \
        build_opener, \
        urlopen, \
        Request
    from urllib import \
        quote_plus, \
        urlencode

    #2to3: json.loads() needs a str not bytes
    def to_str(s): return s
    def to_bytes(bs): return bs
else:
    # Import Python 3 classes
    from urllib.request import \
        HTTPPasswordMgrWithDefaultRealm, \
        HTTPBasicAuthHandler, \
        build_opener, \
        urlopen, \
        Request
    from urllib.parse import \
        quote_plus, \
        urlencode

    #2to3: json.loads() needs a str not bytes
    def to_str(s):
        if type(s) == bytes: return s.decode('utf-8')
        else: return s
    def to_bytes(s): 
        if type(s) == bytes: return s
        else: return s.encode()


def wombat_opener(base_url, user, password):
    """Build an URL opener for accessing Wombat's REST API."""

    # Create a password manager
    password_mgr = HTTPPasswordMgrWithDefaultRealm()
    password_mgr.add_password('WombatOAM - Web Dashboard',
                              base_url, user, password)

    # Create an authentication handler
    handler = HTTPBasicAuthHandler(password_mgr)

    # Create an URL opener
    return build_opener(handler)

def rest_call(url, user, passw, data = None, method = None, format = 'python'):
    """Make a REST call.

    The method defaults to GET if there is no data and POST otherwise.
    The data (when used) will be converted to JSON.

    The result will contain the HTTP status code and the response
    body.

    The response body is expected to be JSON too, and by default
    returned as a python object. By changing the format argument to
    json the raw UTF-8 string can be returned as well.

    """

    if data:
        request_body = to_bytes(json.dumps(data))
        request_headers = {'Content-Type':'application/json'}
    elif method == 'POST':
        request_body = to_bytes('')
        request_headers = {'Content-Type':'application/json'}
    else:
        request_body = None
        request_headers = {}

    request = Request(url, request_body, request_headers)
    if method:
        request.get_method = lambda: method

    auth = '%s:%s' % (user,passw)
    base64_bytes = base64.b64encode(to_bytes(auth))
    base64_string = base64_bytes.decode()
    request.add_header("Authorization", "Basic %s" % base64_string)

    response = urlopen(request)
    response_body = to_str(response.read())
    return (response.code,
            json.loads(response_body) if format != 'json' else response_body)

def quote_path(string):
    """Quote a string to be used as the part of an URL."""
    return quote_plus(string, '')

def quote_args(args):
    """Quote a dictionary of arguments to be used in a REST request."""
    return urlencode(args)
