# Plugin startup sequence

> * **Created by**: Daniel Szoboszlay **on** 13 August 2014

## wo_agent_mgr

The `wo_agent_mgr` application is a conventional Wombat subsystem with
a master server and a middle manager.

The master server at startup loads the plugins from a directory
(configured in a environment variable of the application). A plugin is
defined by a `.plugin` file, similar to a `.app` file:

```erlang
{plugin, mnesia,            % name of the plugin
 [{agent, 'wombat#mnesia'}, % the module implementing the agent behaviour
  {dependencies, []}        % optional list of other modules to be injected
 ]}.
```

The modules referenced from the `.plugin` file are loaded from the
same directory as the `.plugin` file itself.

The middle manager will listen for node up/down events and start a
dedicated `wo_agent_mgr_fsm` process for each node. This dedicated
process will inject the `wo_agent` application's modules to the managed
node.

## wo_agent

The `wo_agent` application is injected into the managed node to
control the plugins there. It will not actually be started as an
application (to minimize impact on the managed system), but it will
define the following process tree:

- Main supervisor (`rest_for_one` strategy, `permanent` children)
  - `wombat#agent` server: for controlling the plugins and relaying
    messages between Wombat and the plugins
  - Plugins supervisor (`one_for_one` strategy, `temporary` children):
    has a separate child for every plugin running on the node;
    isolates faults in plugins
    - Plugin supervisor (`rest_for_one` strategy, `permanent` children)
      - Main plugin process
      - Jobs supervisor (`one_for_one` strategy, `permanent`
        children): started on demand only, if the plugin process wants
        to create jobs.
        - Job processes

Initially of course the plugins supervisor will have no children at
all.

The `wombat#agent` server monitors the node's `wo_agent_mgr_fsm` and
terminates all plugins when connection is lost. As the final step of
the cleanup it stops itself and all supervisors.

## Starting a plugin

Plugins are started one-by-one. The process is initiated by the
`wo_agent_mgr_fsm` once the `wo_agent` is deployed in the managed
node.

1. `wombat#agent` receives the list of modules the plugin uses.
2. `wombat#agent` fetches fetches every module it hasn't yet loaded
   from the `wo_agent_mgr_fsm` and injects them.
3. The plugin's agent module is started (in its proper place in the
   supervision tree). If it fails to start, `wombat#agent` removes the
   plugin (and unloads the code etc.)
4. If the plugin started fine, `wombat#agent` queries the list of
   capabilities from the plugin.
5. Every capability returned by the plugin (or reported later on from
   a dynamic capabilities capability) and every
   metric/alarm/notification event is forwarded to the
   `wo_agent_mgr_fsm` from this point on, tagged with the managed node
   and the plugin's IDs.
6. Every command (activate or deactivate a capability) from Wombat is
   also handed over to `wombat#agent` for relaying to the plugin.
   
   When the options of an already activated capability needs to be
   updated (e.g. polling frequency changed) the capability is
   activated again with the new options.

## Jobs started by a plugin

The plugin process may decide to offload some of its work related to
activated capabilities to helper processes, called jobs. These jobs
have their own place in the supervision tree, so they shall be spawned
using the Wombat agent's API.

- `'wombat#active_job':start(Id, Module, Args)` to start a process
  implementing the `wombat#active_job` behaviour. Wombat only provides
  a message loop for the active job, there are no restrictions on what
  it can do otherwise.
- `'wombat#polling_job':start(Id, Module, Function, Frequency)` to
  start a polling job that will call the specific function with the
  given frequency.

Other job types may be defined later.

## A note on process identification

When a plugin process (either the agent or a job) wants to send a
message to Wombat (through the `wombat#agent`) it needs to:

1. know the PID of `wombat#agent`; and
2. identify itself (the name of the plugin).

These pieces of information could be stored in various places:

- The process dictionary is an obvious option. The jobs can copy over
  this value to the newly started process too conveniently.
- An explicit term passed to the plugin's init function would also
  work. It would be the responsibility of the plugin to hand this term
  over to children in this case.
- The information could be reconstructed from the `$ancestors` field
  of the process dictionary, or maybe from other process properties
  too.

I would be in favour of storing the value in the process dictionary,
but providing a variant of every API function in `wo_agent` that
accepts this term an an argument (similar to how the random number
generator's seed is handled).
