### Wombat Developer Workflow

This page ties together other pages to provide an overview of the development
workflow. This document will grow to include changes and new information. The
page should be kept as lean as possible meaning implementation details of Wombat
etc should not be placed here.


Before you continue you should do the following:

- Read the [ESL Development Handbook](https://github.com/esl/dev-handbook/blob/master/development_handbook.markdown).
- Read the [Wombat Developer Guide](https://github.com/esl/wombat/blob/dev/doc/developers_guide/devguide.md).
- Take a look at the [wombat-docs](https://github.com/esl/wombat-docs/blob/master/docs/index.md)
  project, this is a useful reference to have during development.
- Join the ESL Wombat HipChat group.

The project is managed (compilation, test, release, documentation creation,
starting and stopping Wombat) via a Makefile which provides a thin wrapper around
Rebar (binary packaged with project) as well as some bash commands and
scripts. Not everything is abstracted by Make. Rebar is sometimes invoked
directly. The guide will list commonly used Make and Rebar tasks throughout.

Now that you have an overview it is time to do some development.

1. Create a fork of the
   [Wombat ESL repository](https://github.com/esl/wombat.git), clone it and
   create a new branch for your feature.

2. Switch to a version of Erlang ranging between R15B03 and 17.5.
   [Kerl](https://github.com/yrashk/kerl) can be used to manage multiple
   versions of Erlang.

3. Make sure python3 is installed (needed by eg. to run dialyzer) (`brew install python3`)

4. Start development. The following types of tests can be used.

   ##### Eunit

   Eunit is used for unit testing. These tests should be simple and quick to
   run. They are typically located under `/app/<app_name>/test` e.g.
   `/apps/wo_alarms/test`.

   ```bash
   # To run all the Eunit tests
   make rel       # wombat_plugin test needs the release
   make eunit
   
   # Test single otp app
   ./rebar eunit apps=wo_core
    ```

    ##### Common Test

   Common Test is used for (black-box) system tests. Tests are more complex than
   Eunit tests and typically take longer to execute. Tests are located under
   `/apps/wo_test/test` and end in `_SUITE` e.g. `wo_rest_SUITE`.

   To run the tests ensure Wombat is stopped (`make stop`) and `wombat.config`
   is empty then do the following

   ```bash
   # Runs all common tests
   make rel
   make test_plugin
   make ct
   
   # Run single suite, remove _SUITE from file name
   make ct-suite SUITE=wo_rest
   ```

   There are some dashboard tests, see the *ESL developer handbook* for
   information on running them.

4. Once you have finished your feature and reviewed the code run the following
   and ensure they pass.

   ```bash
   ./rebar xref skip_deps=true
   make analyze
   
   make rel
   make eunit
   make test_plugin
   make ct
   ```
5. Submit a pull request



### Tips
- If you cancel tests by hitting CTRL + C the Erlang VM may not shut down. To
  kill all beam processes started by common test run `make ct_kill_nodes`. To
  kill all beam processes processes run `make hard_stop`.
- When Wombat is running (`make start`) you can open a shell to it by running
  `make attach`.
