This document describes the load test that I performed on the Ecto plugin.

The result is the following: I could not demonstrate that the Ecto plugin would
slow down the test application which performed database writes using Ecto.

## Install Erlang and Elixir

install Erlang 18.3 with kerl (https://github.com/kerl/kerl)
install Elixir 1.3.0 with kiex (https://github.com/taylor/kiex)

## Install PostgreSQL

### Mac

    brew install postgresql # Mac

    createdb

    $ psql
    (http://www.phoenixframework.org/v0.11.0/docs/mix-tasks)
    > CREATE ROLE postgres LOGIN CREATEDB;

### Ubuntu

Install and configure postgres:

    wombat$ sudo apt-get install update
    wombat$ sudo apt-get install postgresql-client
    root# su - postgres
    postgres$ createuser --interactive # create wombat user
    postgres$ createdb wombat  # it should be able to create databases
    wombat$ psql # now it should work

Set a password to wombat (on Ubuntu 14.4, ecto cannot connect without a pw;
on Mac, the connection works without a pw):

    postgres$ psql
    > ALTER USER "wombat" WITH PASSWORD 'wombat';

## Modify the ecto repo

Clone the https://github.com/elixir-ecto/ecto repo and modify the followings:

    diff --git examples/simple/config/config.exs
    index ac2ed25..fe1dd3b 100644
    @@ -3,4 +3,4 @@ use Mix.Config
     config :simple,
       ecto_repos: [Simple.Repo]

    -import_config "#{Mix.env}.exs"
    +import_config "dev.exs"

    diff --git examples/simple/config/dev.exs
    index 5363fcc..54604d8 100644
    @@ -5,4 +5,5 @@ config :simple, Simple.Repo,
       database: "ecto_simple_dev",
       username: "postgres",
       password: "postgres",
    -  hostname: "localhost"
    +  hostname: "localhost",
    +  loggers: []

        diff --git examples/simple/lib/simple.ex
    index 7745781..c82f479 100644
    @@ -66,4 +66,39 @@ defmodule Simple do
              preload: :weather
         Simple.Repo.all(query)
       end
    +
    +  def run_parallel_puts(n, proc_count) do
    +    start = :os.timestamp()
    +    main = self()
    +
    +    # Start the worker processes.
    +    for _ <- 1..proc_count do
    +      pid = spawn_link(fn ->
    +                    run_puts(n)
    +                    send(main, self())
    +                  end)
    +      IO.puts('Started: #{:erlang.pid_to_list(pid)}')
    +    end
    +
    +    # Wait for the worker processes to finish.
    +    for _ <- 1..proc_count do
    +      receive do
    +        pid ->
    +          IO.puts('finished: #{:erlang.pid_to_list(pid)}')
    +      end
    +    end
    +
    +    finish = :os.timestamp()
    +    diff = :timer.now_diff(finish, start)
    +    round(diff / 1000) / 1000
    +  end
    +
    +  def run_puts(0) do
    +    :ok
    +  end
    +
    +  def run_puts(n) when n > 0 do
    +    Simple.Repo.insert!(%Country{name: "Hungary"}, log: true)
    +    run_puts(n - 1)
    +  end
     end

## Compile Ecto sample repo

Copy the modified ecto repo and modify the settings accordingly:

    ...
    config :simple, Simple.Repo,
      adapter: Ecto.Adapters.Postgres,
      database: "wombat",
      username: "wombat",
      password: "wombat",
      hostname: "localhost"

Ensure you can run Elixir:

    . /home/wombat/kerl/18.3/activate
    kiex use 1.3.0

Build the "simple" ecto app
(https://github.com/elixir-ecto/ecto/tree/master/examples/simple):

    mix do deps.get, compile
    mix ecto.create
    mix ecto.migrate
    mix run priv/repo/seeds.exs

## Run load test

The `Simple.run_parallel_puts(insert_per_process, process_count)` call prints
how long it takes to run process_count processes, each performing
insert_per_process insert requests in Ecto.

    $ iex --name ecto@10.100.0.135 --cookie cookie -S mix

    > for i <- 1..10 do Simple.run_parallel_puts(10000,16) end # with Wombat
    [44.297, 43.818, 43.53, 43.205, 43.851, 42.586, 41.616, 43.454, 45.15, 42.729]

    > for i <- 1..10 do Simple.run_parallel_puts(10000,16) end # without Wombat
    [46.977, 45.744, 46.485, 43.195, 43.605, 45.122, 42.979, 44.021, 43.499, 43.795]

Average with Wombat: 43.42
Average without Wombat: 44.54
