How to install Zabbix
---------------------

I installed Zabbix on an Ubuntu 14.04 VM running inside VirtualBox following
[this blog post][blog_post]. I copied the contents of the blog post here, should
it disappear. I also made notes about the cases when I did things differently.

## Network

I set the virtual machine to use a Bridged network instead of the default NAT
network so that my Mac and my virtual Ubuntu can see each other. I used
"ifconfig" on both sides to determine their IP addresses. The VM had a `172.*`
and a `10.*` IP address; the `10.*` IP address was the one that was available
from Mac.

On the VM, I installed `openssh-server` so that I could see that I can connect
to the machine from the outside.

## Hostname

I didn't use a real domain name when the blog post used one. Instead I used the
name of the machine everywhere where I was supposed to use the domain name.

On my Mac, I added a line to the `/etc/hosts` file so that my Mac finds the
machine by its name:

    10.154.1.56 <hostname>

## Apache warning

When I started Apache, I got the following warning:

    $ sudo service apache2 restart
    * Restarting web server apache2
    AH00558: apache2: Could not reliably determine the server's fully qualified
    domain name, using 127.0.1.1. Set the 'ServerName' directive globally to
    suppress this message

I fixed this by creating `/etc/apache2/conf-available/servername.conf` and
adding this line (replace `mydomain` with the domain name):

    ServerName mydomain

Then:

    cd /etc/apache2/conf-enabled
    sudo ln -s ../conf-available/servername.conf .

## The blog post (with a few amendments)

Edit apt source list to add the PPA:

    sudo nano /etc/apt/sources.list

Add the following items at the end of the file:

    # Zabbix Application PPA
    deb http://ppa.launchpad.net/tbfr/zabbix/ubuntu precise main
    deb-src http://ppa.launchpad.net/tbfr/zabbix/ubuntu precise main

Add the PPA's key so that apt-get trusts the source:

    sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys C407E17D5F76A32B

Update /etc/hosts

    127.0.0.1 localhost.localdomain localhost
    52.28.99.149 zabbix.mydomain.com zabbix

Set the server it's hostname

    echo "zabbix.mydomain.com" > /etc/hostname
    hostname -F /etc/hostname

Update and install Zabbix server and required packages.

    sudo apt-get update
    sudo apt-get install zabbix-server-mysql php5-mysql zabbix-frontend-php

Memorize the root password you have chosen, you'll need it later. Next edit the
Zabbix configuration.

    sudo nano /etc/zabbix/zabbix_server.conf

Adjust the following values and make a note of the password you've chosen.
You'll need it later too.

    DBName=zabbix
    DBUser=zabbix
    DBPassword=your_chosen_password_here

Also memorize the chosen password because you will need it when setting the
rights in the MySQL commands.

Unzip the mysql data for importing them into the database you'll create in the
next step.

    cd /usr/share/zabbix-server-mysql/
    sudo gunzip *.gz

Login to MySql using the root password.

    mysql -u root -p

Create a user for Zabbix (and a database) that matches the information we
entered in the `/etc/zabbix/zabbix_server.conf` file.

    create user 'zabbix'@'localhost' identified by 'your_chosen_password_here';
    create database zabbix;
    grant all privileges on zabbix.* to 'zabbix'@'localhost'; flush privileges;
    exit;

Next we'll import the schemas into the newly created database.

    mysql -u zabbix -p zabbix < schema.sql
    mysql -u zabbix -p zabbix < images.sql
    mysql -u zabbix -p zabbix < data.sql

Edit a few PHP settings by modifying the php.ini file.

    sudo nano /etc/php5/apache2/php.ini

Search and adjust the following data. (Or add them if they can't be found.)

    post_max_size = 16M
    max_execution_time = 300
    max_input_time = 300
    date.timezone = "Europe/Amsterdam"

Next copy the example config to the /etc/zabbix directory. Then make the
necessary adjustments too.

    sudo cp /usr/share/doc/zabbix-frontend-php/examples/zabbix.conf.php.example /etc/zabbix/zabbix.conf.php
    sudo nano /etc/zabbix/zabbix.conf.php

    $DB['DATABASE'] = 'zabbix';
    $DB['USER'] = 'zabbix';
    $DB['PASSWORD'] = 'your_chosen_password_here'

Then copy the example apache config to the /etc/apache2/conf-available/
directory to make Zabbix and Apache work together.

    sudo cp /usr/share/doc/zabbix-frontend-php/examples/apache.conf /etc/apache2/conf-available/zabbix.conf
    sudo a2enconf zabbix.conf
    sudo a2enmod alias
    sudo service apache2 restart
    sudo nano /etc/default/zabbix-server

Go to the bottom and adjust the `START` property to read `yes`:

    START=yes

And start the Zabbix server.

    sudo service zabbix-server start

If you would like to have zabbix running on the root of the domain instead of
/zabbix execute the following;

    sudo nano /etc/apache2/sites-available/000-default.conf

And set the `DocumentRoot` like this;

    DocumentRoot /usr/share/zabbix

Then restart Apache.

### Installing Zabbix agent(s) on server machines.

    sudo apt-get update
    sudo apt-get install zabbix-agent
    sudo nano /etc/zabbix/zabbix_agentd.conf

Look for: `Server=localhost` (or `Server=127.0.0.1`) and change `localhost` with
the IP address of your Zabbix install.

Save and close the file and restart the agent software:

    sudo service zabbix-agent restart

In your web browser, navigate to your Zabbix server's IP address followed by
`/zabbix`: `http://xx.xx.xx.xx/zabbix`

Use the following default credentials to login.

    Username = admin
    Password = zabbix

When you have logged in, click on the "Configuration" button, and then "Hosts"
in the top navigation bar.

Click on the name of the server (by default, this should be "Zabbix server").
This will open the host configuration page. Adjust the Hostname to reflect the
hostname of your Zabbix server (this should be the same hostname you entered in
the agent configuration for the server machine). At the bottom, change the
"Status" property to "Monitored". Click save.

You will be dropped back to the Hosts configuration page. Re-click on the
hostname again. This time, click on the "Clone" button at the bottom of the
page. We will configure this to reflect the settings of the client machine.
Change the hostname and the IP address to reflect the correct information from
the client agent file.

In the "groups" section, select "Zabbix servers" and click the right arrow icon
to remove that group. Select the "Linux servers" and click the left arrow icon
to add that group setting.

Click "Save" at the bottom. After a few minutes, you should be able to see both
computers by going to "Monitoring" and then clicking on "Latest data" There
should be information for both the server and client machines populating.

[blog_post]: https://thedutchlab.com/en/news/installing-zabbix-on-ubuntu-14-04. 

