Services REST interface
=======================

GET requests for getting information about services:

- [Get all configurator services available for a node](#get-all-configurator-services-available-for-a-node)
- [Get all configurator services available for a node family](#get-all-configurator-services-available-for-a-node-family)
- [Get all explorer services available for a node](#get-all-explorer-services-available-for-a-node)
- [Get all explorer services available for a node family](#get-all-explorer-services-available-for-a-node-family)
- [Get all executor services available for a node](#get-all-executor-services-available-for-a-node)
- [Get all executor services available for a node family](#get-all-executor-services-available-for-a-node-family)

POST requests to initiate or stop a request:

- [Initiate a new service request](#initiate-a-new-service-request)
- [Initiate a feature request](#initiate-a-feature-request)
- [Stop a running request](#stop-a-running-request)
- [Initiate new process monitors](#initiate-new-process-monitors)

GET requests for getting information about requests:

- [Get information about a request](#get-information-about-a-request)
- [Get all running request per node](#get-all-running-request-per-node)
- [Get all running request per node family](#get-all-running-request-per-node-family)
- [Get all finished requests per node](#get-all-finished-requests-per-node)
- [Get all finished requests per node family](#get-all-finished-requests-per-node-family)

Get all configurator services available for a node
--------------------------------------------------

### Definition

```
GET /api/service/service-info/configurator/node/NODE_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/service/service-info/configurator/node/5b9625f2-2fd1-4311-92aa-3477b33d11d7"
```

### Example response

```json
{
  "service_infos": [
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAMY29uZmlndXJhdG9yZAALZ2V0X2NvbmZpZ3NyAANkABB3b21iYXRAMTI3LjAuMC4xAgAANN4AAAAAAAAAAA==",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Display an application's configuration parameters",
      "description": "Retrieve all configuration parameters of the given application.",
      "options": [
        {
          "name": "application",
          "label": "Application",
          "type": "enum",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [
            "bear",
            "compiler",
            "crypto",
            "elarm",
            "exometer",
            "folsom",
            "goldrush",
            "gproc",
            "kernel",
            "lager",
            "os_mon",
            "sasl",
            "stdlib",
            "syntax_tools",
            "wo_test",
            "wombat_plugin"
          ],
          "listitemType": []
        }
      ]
    },
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAMY29uZmlndXJhdG9yZAALc2V0X2NvbmZpZ3NyAANkABB3b21iYXRAMTI3LjAuMC4xAgAANOYAAAAAAAAAAA==",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Set a configuration parameter",
      "description": "Set a configuration parameter of the given application.",
      "options": [
        {
          "name": "application",
          "label": "Application",
          "type": "enum",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [
            "bear",
            "compiler",
            "crypto",
            "elarm",
            "exometer",
            "folsom",
            "goldrush",
            "gproc",
            "kernel",
            "lager",
            "os_mon",
            "sasl",
            "stdlib",
            "syntax_tools",
            "wo_test",
            "wombat_plugin"
          ],
          "listitemType": []
        },
        {
          "name": "changes",
          "label": "List of configuration parameters to change",
          "type": "list",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": [
            {
              "name": "configParamName",
              "label": "Configuration parameter name",
              "type": "string",
              "default": "",
              "enabled": true,
              "mandatory": true,
              "values": [],
              "listitemType": []
            },
            {
              "name": "configParamValue",
              "label": "The value to be set for the configuration parameter",
              "type": "string",
              "default": "",
              "enabled": true,
              "mandatory": true,
              "values": [],
              "listitemType": []
            },
            {
              "name": "persist",
              "label": "Shall the change be persistent?",
              "type": "enum",
              "default": "No",
              "enabled": false,
              "mandatory": true,
              "values": [
                "Yes",
                "No"
              ],
              "listitemType": []
            }
          ]
        }
      ]
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_ID** | **Required.** The identifier of the node.

### Returns

A `service_infos` object containing the list of available services for the given
node.

Get all configurator services available for a node family
---------------------------------------------------------

### Definition

```
GET /api/service/service-info/configurator/node-family/NODE_FAMILY_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/service/service-info/configurator/node-family/12a74b0b-2d50-4270-a90c-2b5e50c2e0c8"
```

### Example response

```json
{
  "service_infos": [
    {
      "serviceId": "g2gFZAAGZmFtaWx5bQAAACQxMmE3NGIwYi0yZDUwLTQyNzAtYTkwYy0yYjVlNTBjMmUwYzhkAAxjb25maWd1cmF0b3JkAAtzZXRfY29uZmlnc3IAA2QAEHdvbWJhdEAxMjcuMC4wLjECAAA09gAAAAAAAAAA",
      "objectType": "node-family",
      "objectId": "12a74b0b-2d50-4270-a90c-2b5e50c2e0c8",
      "isInternal": false,
      "label": "Change global config's value",
      "description": "Change the value of a global config on nodes ['test1@127.0.0.1','test2@127.0.0.1','test3@127.0.0.1'].",
      "options": [
        {
          "name": "application",
          "label": "Application",
          "type": "string",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        },
        {
          "name": "changes",
          "label": "List of configuration parameters to change",
          "type": "list",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": [
            {
              "name": "configParamName",
              "label": "The name of the configuration parameter",
              "type": "string",
              "default": "",
              "enabled": true,
              "mandatory": true,
              "values": [],
              "listitemType": []
            },
            {
              "name": "configParamValue",
              "label": "The value to be set for the configuration parameter",
              "type": "string",
              "default": "",
              "enabled": true,
              "mandatory": true,
              "values": [],
              "listitemType": []
            }
          ]
        }
      ]
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_FAMILY_ID** | **Required.** The identifier of the node family.

### Returns

A `service_infos` object containing the list of available services for the given
node family.

Get all explorer services available for a node
----------------------------------------------

### Definition

```
GET /api/service/service-info/explorer/node/NODE_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/service/service-info/explorer/node/5b9625f2-2fd1-4311-92aa-3477b33d11d7"
```

### Example response

```json
{
  "service_infos": [
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAIZXhwbG9yZXJkABJwcm9jZXNzX2RpY3Rpb25hcnlyAANkABB3b21iYXRAMTI3LjAuMC4xAgAANM4AAAAAAAAAAA==",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Process dictionary",
      "description": "Show process dictionary",
      "options": [
        {
          "name": "Name",
          "label": "Registered name",
          "type": "string",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        }
      ]
    },
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAIZXhwbG9yZXJkAAxwcm9jZXNzX2luZm9yAANkABB3b21iYXRAMTI3LjAuMC4xAgAANMoAAAAAAAAAAA==",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Process info",
      "description": "Show process info",
      "options": [
        {
          "name": "Name",
          "label": "Registered name",
          "type": "string",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        }
      ]
    },
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAIZXhwbG9yZXJkABBwcm9jZXNzX21lc3NhZ2VzcgADZAAQd29tYmF0QDEyNy4wLjAuMQIAADTMAAAAAAAAAAA=",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Process messages",
      "description": "Show process messages",
      "options": [
        {
          "name": "Name",
          "label": "Registered name",
          "type": "string",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        }
      ]
    },
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAIZXhwbG9yZXJkABNwcm9jZXNzX3N0YWNrX3RyYWNlcgADZAAQd29tYmF0QDEyNy4wLjAuMQIAADTSAAAAAAAAAAA=",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Process stack trace",
      "description": "Show process stack trace",
      "options": [
        {
          "name": "Name",
          "label": "Registered name",
          "type": "string",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        }
      ]
    },
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAIZXhwbG9yZXJkAA1wcm9jZXNzX3N0YXRlcgADZAAQd29tYmF0QDEyNy4wLjAuMQIAADTQAAAAAAAAAAA=",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Process state",
      "description": "Show process state",
      "options": [
        {
          "name": "Name",
          "label": "Registered name",
          "type": "string",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        }
      ]
    },
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAIZXhwbG9yZXJkABJvYnNlcnZlcl9wcm9jZXNzZXNyAANkABB3b21iYXRAMTI3LjAuMC4xAgAAMqgAAAAAAAAAAA==",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Processes",
      "description": "Display data about running processes.",
      "options": [
        {
          "name": "Interval",
          "label": "Refresh interval in seconds",
          "type": "number",
          "default": "10",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        },
        {
          "name": "Number of lines",
          "label": "Number of listed processes",
          "type": "number",
          "default": "20",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        },
        {
          "name": "Order by",
          "label": "Order by column",
          "type": "enum",
          "default": "Reds",
          "enabled": true,
          "mandatory": true,
          "values": [
            "Pid",
            "Name or Initial Func",
            "Reds",
            "Memory",
            "MsgQ",
            "Current Function"
          ],
          "listitemType": []
        },
        {
          "name": "Order direction",
          "label": "Order direction",
          "type": "enum",
          "default": "Descending",
          "enabled": true,
          "mandatory": true,
          "values": [
            "Ascending",
            "Descending"
          ],
          "listitemType": []
        }
      ]
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_ID** | **Required.** The identifier of the node.

### Returns

A `service_infos` object containing the list of available services for the given
node.

Get all explorer services available for a node family
-----------------------------------------------------

### Definition

```
GET /api/service/service-info/explorer/node-family/NODE_FAMILY_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/service/service-info/explorer/node-family/12a74b0b-2d50-4270-a90c-2b5e50c2e0c8"
```

### Example response

```json
{
  "service_infos": []
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_FAMILY_ID** | **Required.** The identifier of the node family.

### Returns

A `service_infos` object containing the list of available services for the given
node family.

Get all executor services available for a node
----------------------------------------------

### Definition

```
GET /api/service/service-info/executor/node/NODE_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/service/service-info/executor/node/5b9625f2-2fd1-4311-92aa-3477b33d11d7"
```

### Example response

```json
{
  "service_infos": [
    {
      "serviceId": "g1AAAAB3eJwtxkEOgjAQRuE/qIlLFx6BbY0zAzbsvAlpmSFuSpMC0WtwY0z0fZv3OimOU1ZLAOo2dg9uR3Y8KrlGiFzHIThpvI8iSqRecbaPDeuSi+L6W+vX2Uo/5JTCpAUHxeWdUwzLk9jf7l9UAbLh3w52ZB63",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Execute user command",
      "description": "Execute user command on the managed node.",
      "options": [
        {
          "name": "Expr",
          "label": "Expression to be evaluated",
          "type": "string",
          "default": "",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        }
      ]
    },
    {
      "serviceId": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAIZXhlY3V0b3JkAAhmb3JjZV9nY3IAA2QAEHdvbWJhdEAxMjcuMC4wLjECAAA02wAAAAAAAAAA",
      "objectType": "node",
      "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
      "isInternal": false,
      "label": "Garbage Collect Processes",
      "description": "Force garbage collection on all processes.",
      "options": [
        {
          "name": "GroupSize",
          "label": "Number of processes garbage collectedin a bunch",
          "type": "number",
          "default": "1000",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        },
        {
          "name": "GroupPause",
          "label": "Pause between process GCs in millisecs",
          "type": "number",
          "default": "100",
          "enabled": true,
          "mandatory": true,
          "values": [],
          "listitemType": []
        }
      ]
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_ID** | **Required.** The identifier of the node.

### Returns

A `service_infos` object containing the list of available services for the given
node.

Get all executor services available for a node family
-----------------------------------------------------

### Definition

```
GET /api/service/service-info/executor/node-family/NODE_FAMILY_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/service/service-info/executor/node-family/12a74b0b-2d50-4270-a90c-2b5e50c2e0c8"
```

### Example response

```json
{
  "service_infos": []
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_FAMILY_ID** | **Required.** The identifier of the node family.

### Returns

A `service_infos` object containing the list of available services for the given
node family.

Initiate a new service request
------------------------------

### Definition

```
POST /api/request/service_request
```

### Example request

```
curl -X POST "http://127.0.0.1:8080/api/request/service_request" \
     -H "Content-Type: application/json" \
     -d '{"id": "g2gFZAAEbm9kZW0AAAAkNWI5NjI1ZjItMmZkMS00MzExLTkyYWEtMzQ3N2IzM2QxMWQ3ZAAMY29uZmlndXJhdG9yZAALc2V0X2NvbmZpZ3NyAANkABB3b21iYXRAMTI3LjAuMC4xAgAANOYAAAAAAAAAAA==", "arguments": {"application": "lager", "changes": [{"configParamName": "crash_log_count", "configParamValue": "10", "persist": "No"}]}}'
```

### Example response

```json
{
  "requestId": "108f6333-d518-4017-8b0a-da245d1ef2a8",
  "label": "Set a configuration parameter"
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**id** | **Required.** The identifier of service (serviceId) as returned by the GET requests above.
**arguments** | **Required.** The arguments to be passed to the service. Each value should be either a string or a list containing string→string objects.

### Returns

Information about the request that was initiated.

Initiate a feature request
--------------------------

When a service responds by sending data to WombatOAM (which is displayed to the
user), this data may contain further actions that can be executed by the user.

Let's assume that we have a "Processes" request, whose response to the [Get
information about a request](#get-information-about-a-request) HTTP request
contains a table of the top processes on the managed node, with some additional
information.

The cells in the first column in this table are links because they provide a
local menu, which shows all the actions that can be performed in this cell.
Each such action defines a new potential request. This "initiate a
feature request" HTTP request is about initiating such a feature request.

The JSON response behind this Dashboard table is the following:

```
  "data": [
    [
      {
        "data": "<0.3.0>",
        "actions": [
          {
            "label": "Terminate process",
            "featureRequest": {
              "featureName": "kill_process",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAACaAJtAAAAA1BpZG0AAAAHPDAuMy4wPmgCbQAAAAZSZWFzb25tAAAABGtpbGxq"
            }
          },
          ... // further actions in this cell
        ]
      },
      ... // other columns
    ],
    ... // other rows
  ]
```

When the user selects an action, that action shall be sent to WombatOAM using the
"Initiate a feature request" HTTP request without being changed. The "arguments"
parameter contains the arguments needed by the new feature request (in this case
the pid of the selected process) in an encoded format.

### Definition

```
POST /api/request/feature_request
```

### Example request

```
curl -X POST "http://127.0.0.1:8080/api/request/feature_request" \
     -H "Content-Type: application/json" \
     -d '{"featureName": "set_as_global", "objectType": "node-family", "objectId": null, "node": "test1@127.0.0.1", "arguments": "g2wAAAACaAJtAAAAC2FwcGxpY2F0aW9ubQAAAAVsYWdlcmgCbQAAAA9jb25maWdQYXJhbU5hbWVtAAAAD2NyYXNoX2xvZ19jb3VudGo="}'
```

### Example response

```json
{
  "requestId": "6dee9dab-2266-45e2-8fcb-f8fe500ae8a3",
  "label": "Set application parameter as global"
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**featureName** | **Required.** The name of the feature to be initiated.
**objectType** | **Required.** Either "node" or "node-family". It specified whether the action should be performed on a node or a node family.
**objectId** | **Required.** (string or `null`) This field can be used to specify the id of the node or node family on which the action should be performed.
**node** | **Optional.**  (string or `null`) This field can be used to specify the name of the node on which the action should be performed if `objectId` is `null`. Either `objectId` or `node` has to be not `null`.
**arguments** | **Required.** The arguments to be given to the feature. This field is received in an encoded format from `/api/request/request-info` and shall be supplied here unchanged.

### Returns

Information about the request that was initiated.

Stop a running request
----------------------

Stop a request. Note that only interruptible requests can be stopped, such as
periodically listing information about the processes.

### Definition

```
POST /api/request/stop/REQUEST_ID
```

### Example request

```
curl -X POST "http://127.0.0.1:8080/api/request/stop/108f6333-d518-4017-8b0a-da245d1ef2a8" \
     -H "Content-Type: application/json"
```

### Example response (success)

In case of success, the response body is empty.

### Example response (error)

```json
{
  "error": "{no_such_child,<<\"108f6333-d518-4017-8b0a-da245d1ef2a8\">>}"
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**REQUEST\_ID** | **Required.** The identifier of the request to be stopped.

### Returns

An empty response.

Initiate new process monitors
----------------------

Start monitoring given processes on all or given nodes. Without a list of nodes the processes will be monitored on all nodes.

### Definition

```
POST /api/config/monitor/PROCESSES
```

### Example requests

```
Version 1:
curl -X POST "http://localhost:8080/api/config/monitor" 
  -H "Content-Type: application/json" 
  -d '{"processes": ["process_to_monitor"]}'

Version 2:
curl -X POST "http://localhost:8080/api/config/monitor" 
  -H "Content-Type: application/json" 
  -d '{"processes": ["process_to_monitor"], "nodes": ["wombat@127.0.0.1"]}'
```

### Example response (success)

In case of success, the response body is contains a true value.

### Example response (error)

```json
{
  "error": "{missing_key, <<\"Missing -processes- key from body\">>}"
}
```
```json
{
  "error": 
  "{
    success: false,
    error: <<\"Only admin users are allowed to do this request\">>,
    reason: not_authorized
  }"
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**PROCESSES** | **Required.** The list of processes must be given in order to start monitoring. List of nodes can be left out.

### Returns

An empty response.

Get information about a request
-------------------------------

### Definition

```
GET /api/request/request-info/REQUEST_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/request/request-info/0cbde1cb-7d3e-4e98-9c20-86d14d9dd8e4"
```

### Example response

```json
{
  "requestId": "0cbde1cb-7d3e-4e98-9c20-86d14d9dd8e4",
  "label": "Processes",
  "objectType": "node",
  "objectId": "5b9625f2-2fd1-4311-92aa-3477b33d11d7",
  "startDate": 1459016287788,
  "endDate": null,
  "arguments": [
    {
      "label": "Refresh interval in seconds",
      "value": "10"
    },
    {
      "label": "Number of listed processes",
      "value": "2"
    },
    {
      "label": "Order by column",
      "value": "Reds"
    },
    {
      "label": "Order direction",
      "value": "Descending"
    }
  ],
  "infoMessages": [],
  "displayInfo": {
    "dataStructure": "table",
    "label": "Processes",
    "tableHeaders": [
      "Pid",
      "Name or Initial Func",
      "Reds",
      "Memory",
      "MsgQ",
      "Current Function"
    ],
    "isInteractive": true
  },
  "streamInfo": {
    "isPeriodic": true,
    "isInterruptible": true
  },
  "data": [
    [
      {
        "data": "<0.3.0>",
        "actions": [
          {
            "label": "Terminate process",
            "featureRequest": {
              "featureName": "kill_process",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAACaAJtAAAAA1BpZG0AAAAHPDAuMy4wPmgCbQAAAAZSZWFzb25tAAAABGtpbGxq"
            }
          },
          {
            "label": "Process info",
            "featureRequest": {
              "featureName": "process_info",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAHPDAuMy4wPmo="
            }
          },
          {
            "label": "Process messages",
            "featureRequest": {
              "featureName": "process_messages",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAHPDAuMy4wPmo="
            }
          },
          {
            "label": "Process dictionary",
            "featureRequest": {
              "featureName": "process_dictionary",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAHPDAuMy4wPmo="
            }
          },
          {
            "label": "Process state",
            "featureRequest": {
              "featureName": "process_state",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAHPDAuMy4wPmo="
            }
          },
          {
            "label": "Process stack trace",
            "featureRequest": {
              "featureName": "process_stack_trace",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAHPDAuMy4wPmo="
            }
          }
        ]
      },
      {
        "data": "erl_prim_loader",
        "actions": []
      },
      {
        "data": "417433294",
        "actions": []
      },
      {
        "data": "62848",
        "actions": []
      },
      {
        "data": "0",
        "actions": []
      },
      {
        "data": "{erl_prim_loader,loop,3}",
        "actions": []
      }
    ],
    [
      {
        "data": "<0.16279.12>",
        "actions": [
          {
            "label": "Terminate process",
            "featureRequest": {
              "featureName": "kill_process",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAACaAJtAAAAA1BpZG0AAAAMPDAuMTYyNzkuMTI+aAJtAAAABlJlYXNvbm0AAAAEa2lsbGo="
            }
          },
          {
            "label": "Process info",
            "featureRequest": {
              "featureName": "process_info",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAMPDAuMTYyNzkuMTI+ag=="
            }
          },
          {
            "label": "Process messages",
            "featureRequest": {
              "featureName": "process_messages",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAMPDAuMTYyNzkuMTI+ag=="
            }
          },
          {
            "label": "Process dictionary",
            "featureRequest": {
              "featureName": "process_dictionary",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAMPDAuMTYyNzkuMTI+ag=="
            }
          },
          {
            "label": "Process state",
            "featureRequest": {
              "featureName": "process_state",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAMPDAuMTYyNzkuMTI+ag=="
            }
          },
          {
            "label": "Process stack trace",
            "featureRequest": {
              "featureName": "process_stack_trace",
              "objectType": "node",
              "objectId": null,
              "node": "test1@127.0.0.1",
              "arguments": "g2wAAAABaAJtAAAAA1BpZG0AAAAMPDAuMTYyNzkuMTI+ag=="
            }
          }
        ]
      },
      {
        "data": "{wombat_plugin,init,1}",
        "actions": []
      },
      {
        "data": "65862347",
        "actions": []
      },
      {
        "data": "3147352",
        "actions": []
      },
      {
        "data": "0",
        "actions": []
      },
      {
        "data": "{gen_server,loop,6}",
        "actions": []
      }
    ]
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**REQUEST\_ID** | **Required.** The identifier of the request.

### Returns

Information about the request, including its last stream data.

Get all running request per node
--------------------------------

### Definition

```
GET /api/request/node/NODE_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/request/node/5b9625f2-2fd1-4311-92aa-3477b33d11d7"
```

### Example response

```json
{
  "requests": [
    {
      "requestId": "eb93644c-5dd8-4b53-ade2-f4024ea7b397",
      "label": "Processes"
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_ID** | **Required.** The identifier of the node.

### Returns

Basic information about the requests running on the given node.

Get all running request per node family
---------------------------------------

### Definition

```
GET /api/request/node-family/NODE_FAMILY_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/request/node-family/12a74b0b-2d50-4270-a90c-2b5e50c2e0c8"
```

### Example response

```json
{
  "requests": [
    {
      "requestId": "84d829d1-453c-4b45-aa51-c0ab670b9b98",
      "label": "List global configs"
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_FAMILY_ID** | **Required.** The identifier of the node family.

### Returns

Basic information about the requests running on the given node family.

Get all finished requests per node
----------------------------------

### Definition

```
GET /api/request/history/node/NODE_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/request/history/node/5b9625f2-2fd1-4311-92aa-3477b33d11d7"
```

### Example response

```json
{
  "requests": [
    {
      "requestId": "b2558d43-f118-4f92-92ea-e9fd29de9bb7",
      "label": "Processes"
    },
    {
      "requestId": "17fa01c5-1fa2-4a41-b288-f7cf5131bc08",
      "label": "Display an application's configuration parameters"
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_ID** | **Required.** The identifier of the node.

### Returns

Basic information about the requests finished on the given node family.

Get all finished requests per node family
-----------------------------------------

### Definition

```
GET /api/request/history/node-family/NODE_FAMILY_ID
```

### Example request

```
curl -X GET "http://127.0.0.1:8080/api/request/history/node-family/12a74b0b-2d50-4270-a90c-2b5e50c2e0c8"
```

### Example response

```json
{
  "requests": [
    {
      "requestId": "b2558d43-f118-4f92-92ea-e9fd29de9bb7",
      "label": "Processes"
    },
    {
      "requestId": "17fa01c5-1fa2-4a41-b288-f7cf5131bc08",
      "label": "Display an application's configuration parameters"
    }
  ]
}
```

### Arguments

**Argument** | **Description**
-------------|----------------
**NODE_FAMILY_ID** | **Required.** The identifier of the node family.

### Returns

Basic information about the requests finished on the given node.
