## application plugin

### Description

The `application` plugin reports information about the OTP applications present
on the managed node that the user can see on the Topology page.
It reports the following information for each loaded application:

 * Name: the name of the application.
 * Description: the short description of the application.
 * Version: the version of the application.
 * Running:
   * The PID of the process started by the application callback if the
     application has a callback module and is running.
   * `library` if the application has no callback, but has been started
     and is running.
   * `Loaded, but not started` if the application has not been started yet.
   * `crashed` if the application has been started, but is not running
     currently. This usually indicates an error on the managed system.
   * `unknown` if the application has been not started, has no callback
     module, but is running. This indicates an error on the managed system.
 * Type: The type of the application (`permanent`, `transient`, `temporary` or
   `unknown`).
    * If a `permanent` application terminates, all other applications and the
     entire Erlang node are also terminated.
    * If a `transient` application terminates with `Reason` == `normal`, this
      is reported but no other applications are terminated.
    * If a `transient` application terminates abnormally, all other applications
      and the entire Erlang node are also terminated.
    * If a `temporary` application terminates, this is reported but no other
     applications are terminated.
    * If the application has not been started yet, its type is `unknown`.

   It is essentially the second parameter of `application:start/2`.
   If the application is running, but has been not started, `Error: application
   running, but not started!` is shown. This indicates an error on the managed
   system.
   
 * Configuration: the list of the environment variables of the application
   together with their current values.

### Applications it depends on

`kernel`

### Modules

`wombat_plugin_application`

### Reports

The plugin doesn't report any metrics or alarms. It sends a log message
when an error is detected in the application status (started, but not running;
running, but not started). It is used for 
displaying information about the OTP applications present on the managed node.

Note: If this plugin is turned off, any values that change will not be
updated on the dashboard.

### Configuration options

`app_info_period`

The period (in ms.) used to report a message with the last status of the 
applications and their environment variables.

#### Example wombat.config entry

```erlang
{set, wo_plugins, plugins, application, app_info_period, 20000},
```

<!--
### Usage

#### Resource use

The resource usage of this plugin is rather low, since it simple performs a check
of the application status. Also, the `app_info_period` value should be rather high
so that this check is not performed too often.

#### Stopping monitoring

Once a new node is added to WombatOAM, this plugin will be stated and a first report
with the application status will be generated and sent to WombatOAM. If the user then
turns the plugin off, the values seen on the dashboard will not be updated. So if
for example one of the environment variables value is changed, this change will
not be visible on the WombatOAM dashboard.
-->
