## rabbitmq channels plugin

### Description

The `rabbitmq_channels` plugin provides the ability to monitor metrics for individual, active channels. When enabled, this plugin will acquire all active channels from the managed rabbit node and based on its configuration, it generates corresponding metrics for each acquired channel.

All channel metric names follow the following format;

**\<CHANNEL-NAME\> ~ \<METRIC-ATTRIBUTE\>**

where:

**`<CHANNEL-NAME>`** is the native acquired RabbitMQ channel name.

**`<METRIC-ATTRIBUTE>`** is one of the following metric attributes;

- number
- reductions
- consumer_count
- messages_unacknowledged
- messages_unconfirmed
- messages_uncommitted
- acks_uncommitted
- prefetch_count
- global\_prefetch_count

Example of a channel metric would be: `192.168.1.209:55605 -> 192.168.1.211:5672 (4) ~ consumer_count`.



### Supported versions.

Official stable Rabbit releases newer than 2.3.0 are supported.

### Applications it depends on

`rabbit`

### Modules

- `wombat_plugin_rabbitmq_channels`
- `wombat_plugin_rabbitmq_metrics`

### Configuration options

* `metric `: The metric attribute to be applied and monitored on all channels. Default value: `consumer_count`.

* `refresh_interval`: Time period after which new metric values are acquired from the rabbit node. Default value: `1000`.

* `limit `: Maximum of the number channel metrics the plugin is allowed to handle on each refresh interval. Default value: `1000`.

#### Example wombat.config entry

```erlang
%% Set the metric attribute
{set, wo_plugins, plugins, rabbitmq_channels, metric, channels}.

%% Change refresh interval after which metric values are refreshed
{set, wo_plugins, plugins, rabbitmq_channels, refresh_interval, 1500}.

%% Limit the maximum number of processed channel metrics
{set, wo_plugins, plugins, rabbitmq_channels, limit, 1500}.

```

### Metrics reported

If the plugin is restarted, all these metrics are reset.

#### Channel metrics:

* **\<CHANNEL-NAME\> ~ number** :
  Tags: `dev`

  The identity and count number of the channel **\<CHANNEL-NAME\>**.

* **\<CHANNEL-NAME\> ~ reductions** :
  Tags: `dev`

  The number of reductions executed by the channel.

* **\<CHANNEL-NAME\> ~ consumer_count** :
  Tags: `dev`

  The number of active consumers on the channel.

* **\<CHANNEL-NAME\> ~ messages_unacknowledged** :
  Tags: `dev`

  The number of unacknowledged messages on the channel.

* **\<CHANNEL-NAME\> ~ messages_unconfirmed** :
  Tags: `dev`

  The number of unconfirmed messages on the channel.

* **\<CHANNEL-NAME\> ~ messages_uncommitted** :
  Tags: `dev`

  The number of uncommitted messages on the channel.

* **\<CHANNEL-NAME\> ~ acks_uncommitted** :
  Tags: `dev`

  The number of uncommitted message acknowledgements on the channel.

* **\<CHANNEL-NAME\> ~ prefetch_count** :
  Tags: `dev`

  The channel prefetch count limit.
  
* **\<CHANNEL-NAME\> ~ global\_prefetch_count** :
  Tags: `dev`

  The global channel prefetch count limit.