## rabbitmq connections plugin

### Description

The `rabbitmq_connections` plugin provides the ability to monitor metrics for individual, active connections. When enabled, this plugin will acquire all active connections from the managed rabbit node and based on its configuration, it generates corresponding metrics for each acquired connection.

All connection metric names follow the following format;

**\<CONNECTION-NAME\> ~ \<METRIC-ATTRIBUTE\>**

where:

**`<CONNECTION-NAME>`** is the native acquired RabbitMQ connection name.

**`<METRIC-ATTRIBUTE>`** is one of the following metric attributes;

- frame_max
- channel_max
- recv_oct
- recv_cnt
- send_oct
- messages_ram
- send_cnt
- send_pend
- channels

Example of a connection metric: `192.168.1.209:55605 -> 192.168.1.211:5672 ~ channels`.



### Supported versions.

Official stable Rabbit releases newer than 2.3.0 are supported.

### Applications it depends on

`rabbit`

### Modules

- `wombat_plugin_rabbitmq_connections`
- `wombat_plugin_rabbitmq_metrics`

### Configuration options

* `metric `: The metric attribute to be applied and monitored on all connections. Default value: `channels`.

* `refresh_interval`: Time period after which new metric values are acquired from the rabbit node. Default value: `1000`.

* `limit `: Maximum of the number connection metrics the plugin is allowed to handle on each refresh interval. Default value: `1000`.

#### Example wombat.config entry

```erlang
%% Set the metric attribute
{set, wo_plugins, plugins, rabbitmq_connections, metric, channels}.

%% Change refresh interval after which metric values are refreshed
{set, wo_plugins, plugins, rabbitmq_connections, refresh_interval, 1500}.

%% Limit the maximum number of processed connection metrics
{set, wo_plugins, plugins, rabbitmq_connections, limit, 1500}.

```

### Metrics reported

If the plugin is restarted, all these metrics are reset.

#### Connection metrics:

* **\<CONNECTION-NAME\> ~ frame_max** :
  Tags: `dev`

  The maximum allowed frame size for connection **\<CONNECTION-NAME\>**.

* **\<CONNECTION-NAME\> ~ channel_max** :
  Tags: `dev`

  The maximum allowed channel number allowed for the connection.

* **\<CONNECTION-NAME\> ~ recv_oct** :
  Tags: `dev`

  The total number of received octets for the connection.

* **\<CONNECTION-NAME\> ~ recv_cnt** :
  Tags: `dev`

  The total number of received packets for the connection.

* **\<CONNECTION-NAME\> ~ send_oct** :
  Tags: `dev`

  The total number of sent octets by the connection.

* **\<CONNECTION-NAME\> ~ send_cnt** :
  Tags: `dev`

  The total number of sent packets for the connection.

* **\<CONNECTION-NAME\> ~ send_pend** :
  Tags: `dev`

  The total number of packets awaiting to be sent by the connection.

* **\<CONNECTION-NAME\> ~ channels** :
  Tags: `dev`

  The total number of channels on the connection.