## rabbitmq queue messages plugin

### Description

The `rabbitmq_queue_messages` plugin currently provides the ability to monitor message rates for individual queues. When enabled, this plugin will acquire all existing from the managed rabbit node and based on its configuration, it generates corresponding message rate metrics for each acquired queue.

All queue metric names follow the following format;

**\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate \<TYPE\> ~ (<\METRIC-PERIOD\>s average)**

where:

**`<QUEUE-NAME>`** is the name of the  acquired RabbitMQ queue,

**`<VHOST>`** is the RabbitMQ VHost the queue belongs to,

**`<TYPE>`** is one of

- get
- get no ack
- deliver
- deliver no ack
- redeliver
- ack
- deliver get

**`<METRIC-PERIOD>`** is 60 (seconds) by default, but can be changed through the configuration;

Example of a queue metric would be: `test-queue ~ / ~ message rate publish ~ (60s average)`.



### Supported versions.

All official 3.7 and later releases are supported and tested.

### Applications it depends on

`rabbit`

### Modules

- `wombat_plugin_rabbitmq_queue_messages`
- `wombat_plugin_rabbitmq_metrics`

### Configuration options

* `metric_period`: specifies the granularity of the samples in milliseconds. It is recommended to set it to a multiple of 5000. The minimum value is 5000. When set to a lower value, it will be ignored and 5000 will be used instead. The default value is 60000, that is 60 seconds.

#### Example wombat.config entry

```erlang
%% Change the metric period
{set, wo_plugins, plugins, rabbitmq_queue_messages, metric_period, 5000}.
```

### Metrics reported

#### Queue message rate metrics:

* **\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate get ~ (<\METRIC-PERIOD\>s average)**
  Tags: `dev`, `op`

  The average rate of Basic.get operations on **\<QUEUE-NAME\>** in the last **\<METRIC-PERIOD\>** seconds.

* **\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate get no ack ~ (<\METRIC-PERIOD\>s average)**
  Tags: `dev`, `op`

  The average rate of Basic.get operations with auto acknowledgement on **\<QUEUE-NAME\>** in the last **\<METRIC-PERIOD\>** seconds.

* **\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate deliver ~ (<\METRIC-PERIOD\>s average)**
  Tags: `dev`, `op`

  The average rate of messages delivered from **\<QUEUE-NAME\>** in the last **\<METRIC-PERIOD\>** seconds.

* **\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate deliver no ack ~ (<\METRIC-PERIOD\>s average)**
  Tags: `dev`, `op`

  The average rate of messages delivered from **\<QUEUE-NAME\>** with auto acknowledgement in the last **\<METRIC-PERIOD\>** seconds.

* **\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate redeliver ~ (<\METRIC-PERIOD\>s average)**
  Tags: `dev`, `op`

  The average rate of messages redelivered from **\<QUEUE-NAME\>** in the last **\<METRIC-PERIOD\>** seconds.

* **\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate ack ~ (<\METRIC-PERIOD\>s average)**
  Tags: `dev`, `op`

  The average rate of message acknowledgements received for delivered messages in **\<QUEUE-NAME\>** in the last **\<METRIC-PERIOD\>** seconds.

* **\<QUEUE-NAME\> ~ \<VHOST\> ~ message rate deliver get ~ (<\METRIC-PERIOD\>s average)**
  Tags: `dev`, `op`

  The average rate of messages delivered with Basic.get operations from **\<QUEUE-NAME\>** in the last **\<METRIC-PERIOD\>** seconds.
